import { ItemsSym, MembersSym, SchemaInit, isCoValueClass, } from "../internal.js";
/** @category Schema definition */
export const Encoders = {
    Date: {
        encode: (value) => value.toISOString(),
        decode: (value) => new Date(value),
    },
};
const optional = {
    ref: optionalRef,
    json() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: "json" };
    },
    encoded(arg) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: { encoded: arg } };
    },
    string: {
        [SchemaInit]: "json",
    },
    number: {
        [SchemaInit]: "json",
    },
    boolean: {
        [SchemaInit]: "json",
    },
    null: {
        [SchemaInit]: "json",
    },
    Date: {
        [SchemaInit]: { encoded: Encoders.Date },
    },
    literal(..._lit) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: "json" };
    },
};
/** @category Schema definition */
export const co = {
    string: {
        [SchemaInit]: "json",
    },
    number: {
        [SchemaInit]: "json",
    },
    boolean: {
        [SchemaInit]: "json",
    },
    null: {
        [SchemaInit]: "json",
    },
    Date: {
        [SchemaInit]: { encoded: Encoders.Date },
    },
    literal(..._lit) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: "json" };
    },
    json() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: "json" };
    },
    encoded(arg) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return { [SchemaInit]: { encoded: arg } };
    },
    ref,
    items: ItemsSym,
    members: MembersSym,
    optional,
};
function optionalRef(arg) {
    return ref(arg, { optional: true });
}
function ref(arg, options) {
    return {
        [SchemaInit]: {
            ref: arg,
            optional: options?.optional || false,
        },
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
    };
}
export function isRefEncoded(schema) {
    return (typeof schema === "object" &&
        "ref" in schema &&
        "optional" in schema &&
        typeof schema.ref === "function");
}
export function instantiateRefEncoded(schema, raw) {
    return isCoValueClass(schema.ref)
        ? schema.ref.fromRaw(raw)
        : schema.ref(raw).fromRaw(raw);
}
//# sourceMappingURL=schema.js.map