export const subscriptionsScopes = new WeakMap();
const TRACE_INVALIDATIONS = false;
export class SubscriptionScope {
    constructor(root, rootSchema, onUpdate) {
        this.scopeID = `scope-${Math.random().toString(36).slice(2)}`;
        this.entries = new Map();
        this.scheduledUpdate = false;
        this.cachedValues = {};
        this.parents = {};
        this.rootEntry = {
            state: "loaded",
            value: root._raw,
            rawUnsub: () => { }, // placeholder
        };
        this.entries.set(root.id, this.rootEntry);
        subscriptionsScopes.set(root, this);
        this.subscriber = root._loadedAs;
        this.scheduleUpdate = () => {
            const value = rootSchema.fromRaw(this.rootEntry.value);
            subscriptionsScopes.set(value, this);
            onUpdate(value);
        };
        this.rootEntry.rawUnsub = root._raw.core.subscribe((rawUpdate) => {
            if (!rawUpdate)
                return;
            this.rootEntry.value = rawUpdate;
            this.scheduleUpdate();
        });
    }
    onRefAccessedOrSet(fromId, accessedOrSetId) {
        // console.log("onRefAccessedOrSet", this.scopeID, accessedOrSetId);
        if (!accessedOrSetId) {
            return;
        }
        this.parents[accessedOrSetId] = this.parents[accessedOrSetId] || new Set();
        this.parents[accessedOrSetId].add(fromId);
        if (!this.entries.has(accessedOrSetId)) {
            const loadingEntry = {
                state: "loading",
                immediatelyUnsub: false,
            };
            this.entries.set(accessedOrSetId, loadingEntry);
            const node = this.subscriber._type === "Account"
                ? this.subscriber._raw.core.node
                : this.subscriber.node;
            void node.loadCoValueCore(accessedOrSetId).then((core) => {
                if (loadingEntry.state === "loading" && loadingEntry.immediatelyUnsub) {
                    return;
                }
                if (core !== "unavailable") {
                    const entry = {
                        state: "loaded",
                        rawUnsub: () => { }, // placeholder
                    };
                    this.entries.set(accessedOrSetId, entry);
                    const rawUnsub = core.subscribe((rawUpdate) => {
                        // console.log("ref update", this.scopeID, accessedOrSetId, JSON.stringify(rawUpdate))
                        if (!rawUpdate)
                            return;
                        this.invalidate(accessedOrSetId);
                        this.scheduleUpdate();
                    });
                    entry.rawUnsub = rawUnsub;
                }
            });
        }
    }
    invalidate(id, fromChild, seen = new Set()) {
        if (seen.has(id))
            return;
        TRACE_INVALIDATIONS &&
            console.log("invalidating", fromChild, "->", id, this.cachedValues[id]);
        delete this.cachedValues[id];
        seen.add(id);
        for (const parent of this.parents[id] || []) {
            this.invalidate(parent, id, seen);
        }
    }
    unsubscribeAll() {
        for (const entry of this.entries.values()) {
            if (entry.state === "loaded") {
                entry.rawUnsub();
            }
            else {
                entry.immediatelyUnsub = true;
            }
        }
        this.entries.clear();
    }
}
//# sourceMappingURL=subscriptionScope.js.map